@@ -1,6 +1,6 @@
 /* errmac.h  -  Utility, debugging, and error checking macros
  *
- * Copyright (c) 1998,2001,2006 Sampo Kellomaki <sampo@iki.fi>, All Rights Reserved.
+ * Copyright (c) 1998,2001,2006,2010 Sampo Kellomaki <sampo@iki.fi>, All Rights Reserved.
  * Copyright (c) 2001-2008 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * This is free software and comes with NO WARRANTY. For licensing
  * see file COPYING in the distribution directory.
@@ -25,7 +25,6 @@
 #include <windows.h>
 #define MS_LONG LONG
 #define MKDIR(d,p) mkdir(d)
-#define pthread_self() GetCurrentThreadId()
 #define GETTIMEOFDAY(tv, tz) ((tv) ? (((tv)->tv_sec = time(0)) && ((tv)->tv_usec = 0)) : -1)
 #define GMTIME_R(secs,stm) do { struct tm* stx_tm = gmtime(&(secs)); if (stx_tm) memcpy(&stm, stx_tm, sizeof(struct tm)); } while(0)   /* *** still not thread safe */
 
@@ -50,7 +49,7 @@ extern int assert_nonfatal;
 extern int trace;   /* this gets manipulated by -v or similar flag */
 #endif
 
-#define TCP_PROTO 6  /* never seen getprotobyname("tcp") return anything else */
+#define TCP_PROTO 6 /* never seen getprotobyname("tcp") return anything else */
 
 /* END CONFIG */
 
@@ -178,11 +177,11 @@ extern int trace;   /* this gets manipulated by -v or similar flag */
 #define CONV_DIGIT(x) ((x) - '0')
 
 /* Original Base64  Len  (x+2) / 3
- * ""       ""        0  2     0(2)
- * 1        WX==      4  3     1(0)
- * 12       WXY=      4  4     1(1)
- * 123      WXYZ      4  5     1(2)
- * 1234     WXYZwx==  8  6     2(0)
+ * ""       ""        0  2     0(2)     Suitable original(packed) sizes = bits
+ * 1        WX==      4  3     1(0)     to avoid padding are
+ * 12       WXY=      4  4     1(1)      3(4) = 24,   6(8)= 48,  9(12)= 72, 12(16)= 96,
+ * 123      WXYZ      4  5     1(2)     15(20)=120, 18(24)=144, 21(28)=168, 24(32)=192,
+ * 1234     WXYZwx==  8  6     2(0)     27(36)=216, 30(40)=240, 33(44)=264, 36(48)=288
  * 12345    WXYZwxy=  8  7     2(1)
  * 123456   WXYZwxyz  8  8     2(2)
  */
@@ -369,8 +368,17 @@ extern int trace;   /* this gets manipulated by -v or similar flag */
 
 /* =============== pthread locking =============== */
 
-#define LOCK(l,lk) if (pthread_mutex_lock(&(l))) NEVERNEVER("DEADLOCK(%s)", (lk))
+#ifdef USE_PTHREAD
+/*#define LOCK_STATIC(l) pthread_mutex_t l = PTHREAD_MUTEX_INITIALIZER  do not use */
+#define LOCK_INIT(l) pthread_mutex_init(&(l), 0)
+#define LOCK(l,lk)   if (pthread_mutex_lock(&(l)))   NEVERNEVER("DEADLOCK(%s)", (lk))
 #define UNLOCK(l,lk) if (pthread_mutex_unlock(&(l))) NEVERNEVER("UNLOCK-TWICE(%s)", (lk))
+#else
+#define LOCK_STATIC(l) 
+#define LOCK_INIT(l)
+#define LOCK(l,lk)
+#define UNLOCK(l,lk)
+#endif
 
 /* =============== file system flocking =============== */
 
@@ -403,6 +411,8 @@ extern char zx_instance[64];
 extern int zx_debug;         /* Defined in zxidlib.c */
 extern char zx_indent[256];  /* Defined in zxidlib.c. *** Locking issues? */
 #if 1
+/* In some scenarios multithreaded access can cause zx_indent to be scrambled.
+ * However, it should not under- or overflow. Thus no lock. */
 #define D_INDENT(s) strncat(zx_indent, (s), sizeof(zx_indent)-1)
 #define D_DEDENT(s) (zx_indent[MAX(0, strlen(zx_indent)-sizeof(s)+1)] = 0)
 #else
