@@ -27,6 +27,7 @@
 #include <zx/zxidconf.h>  /* Default and compile-time configuration options. */
 #include <zx/wsf.h>
 #include <zx/c/zxidvers.h>
+#include <zx/c/zx-e-data.h>
 
 char* help =
 "zxidhlowsf  -  SAML 2.0 SP + WSC CGI - R" ZXID_REL "\n\
@@ -55,12 +56,12 @@ Usage: zxidhlowsf [options]   (when used as CGI, no options can be supplied)\n\
 /* Called by: */
 int main(int argc, char** argv)
 {
-  struct zxid_conf* cf;
-  struct zxid_ses sess;
-  struct zxid_ses* ses;
+  zxid_conf* cf;
+  zxid_ses sess;
+  zxid_ses* ses;
   //struct zx_root_s* r;
   struct zx_e_Envelope_s* env;
-  struct zx_a_EndpointReference_s* epr;
+  zxid_epr* epr;
   char* p;
   char* sid;
   char* nid;
