@@ -1,4 +1,5 @@
 /* zxidlib.c  -  Handwritten functions for implementing common application logic for SP
+ * Copyright (c) 2010 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
  * Copyright (c) 2006-2009 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
  * This is confidential unpublished proprietary source code of the author.
@@ -16,6 +17,7 @@
 #include <string.h>
 #include <stdio.h>
 
+#include "platform.h"
 #include "errmac.h"
 #include "zxid.h"
 #include "zxidconf.h"
@@ -46,7 +48,7 @@ int zxid_version()
  * used for runtime version display. For compile time you
  * should check the value of the ~ZXID_VERSION~ macro. */
 
-/* Called by:  main x9, opt x2, zxid_an_page_cf, zxid_fed_mgmt_cf, zxid_idp_select_zxstr_cf_cgi, zxid_mgmt */
+/* Called by:  main x7, opt x2, zxid_fed_mgmt_cf, zxid_idp_select_zxstr_cf_cgi, zxid_map_bangbang, zxid_mgmt */
 char* zxid_version_str()
 {
   return ZXID_REL " " ZXID_COMPILE_DATE " libzxid (zxid.org)";
@@ -61,8 +63,8 @@ char* zxid_version_str()
  *     bits should be multiple of 24 (3 bytes expands to 4 safe base64 chars)
  * return:: The identifier as zx_str. Caller should eventually free this memory.
  */
-/* Called by:  zxid_add_fed_tok_to_epr x2, zxid_check_fed, zxid_di_query, zxid_idp_sso x2, zxid_mk_a7n, zxid_mk_art_deref, zxid_mk_authn_req, zxid_mk_az, zxid_mk_az_cd1, zxid_mk_dap_query_item, zxid_mk_dap_resquery, zxid_mk_dap_subscription, zxid_mk_dap_test_item, zxid_mk_logout, zxid_mk_logout_resp, zxid_mk_mni, zxid_mk_mni_resp, zxid_mk_saml_resp, zxid_mk_subj, zxid_put_ses, zxid_pw_authn, zxid_wsc_call */
-struct zx_str* zxid_mk_id(struct zxid_conf* cf, char* prefix, int bits)
+/* Called by:  zxid_check_fed, zxid_di_query, zxid_mk_a7n, zxid_mk_art_deref, zxid_mk_authn_req, zxid_mk_az, zxid_mk_az_cd1, zxid_mk_dap_query_item, zxid_mk_dap_resquery, zxid_mk_dap_subscription, zxid_mk_dap_test_item, zxid_mk_logout, zxid_mk_logout_resp, zxid_mk_mni, zxid_mk_mni_resp, zxid_mk_saml_resp, zxid_mk_subj, zxid_mk_transient_nid, zxid_put_ses, zxid_pw_authn, zxid_wsc_prep_secmech, zxid_wsf_decor */
+struct zx_str* zxid_mk_id(zxid_conf* cf, char* prefix, int bits)
 {
   char bit_buf[ZXID_ID_MAX_BITS/8];
   char base64_buf[ZXID_ID_MAX_BITS/6 + 1];
@@ -82,8 +84,8 @@ struct zx_str* zxid_mk_id(struct zxid_conf* cf, char* prefix, int bits)
  * without milliseconds form. Some other softwares are buggy and fail to
  * accept the without milliseconds form. You can change the format at compile time.
  */
-/* Called by:  zxid_mk_a7n x3, zxid_mk_art_deref, zxid_mk_authn_req, zxid_mk_az, zxid_mk_az_cd1, zxid_mk_logout, zxid_mk_logout_resp, zxid_mk_mni, zxid_mk_mni_resp, zxid_mk_saml_resp, zxid_wsc_call */
-struct zx_str* zxid_date_time(struct zxid_conf* cf, time_t secs)
+/* Called by:  zxid_mk_a7n x3, zxid_mk_art_deref, zxid_mk_authn_req, zxid_mk_az, zxid_mk_az_cd1, zxid_mk_logout, zxid_mk_logout_resp, zxid_mk_mni, zxid_mk_mni_resp, zxid_mk_saml_resp, zxid_wsc_prep_secmech, zxid_wsf_decor */
+struct zx_str* zxid_date_time(zxid_conf* cf, time_t secs)
 {
   struct tm t;
   secs += cf->timeskew;
@@ -113,7 +115,7 @@ struct zx_str* zxid_date_time(struct zxid_conf* cf, time_t secs)
  * return:: XML data structure representing the response  */
 
 /* Called by:  zxid_wsc_call */
-struct zx_root_s* zxid_soap_call_envelope(struct zxid_conf* cf, struct zx_str* url, struct zx_e_Envelope_s* env)
+struct zx_root_s* zxid_soap_call_envelope(zxid_conf* cf, struct zx_str* url, struct zx_e_Envelope_s* env)
 {
   struct zx_root_s* r;
   struct zx_str* ss;
@@ -137,7 +139,7 @@ struct zx_root_s* zxid_soap_call_envelope(struct zxid_conf* cf, struct zx_str* u
  * return:: XML data structure representing the response  */
 
 /* Called by:  zxid_pep_az_soap, zxid_soap_call_body */
-struct zx_root_s* zxid_soap_call_hdr_body(struct zxid_conf* cf, struct zx_str* url, struct zx_e_Header_s* hdr, struct zx_e_Body_s* body)
+struct zx_root_s* zxid_soap_call_hdr_body(zxid_conf* cf, struct zx_str* url, struct zx_e_Header_s* hdr, struct zx_e_Body_s* body)
 {
   struct zx_root_s* r;
   struct zx_str* ss;
@@ -161,30 +163,52 @@ struct zx_root_s* zxid_soap_call_hdr_body(struct zxid_conf* cf, struct zx_str* u
  * body::   XML data structure representing the SOAP body
  * return:: XML data structure representing the response  */
 
-/* Called by:  zxid_idp_soap, zxid_sp_deref_art, zxid_sp_soap */
-struct zx_root_s* zxid_soap_call_body(struct zxid_conf* cf, struct zx_str* url, struct zx_e_Body_s* body)
+/* Called by:  zxid_as_call_ses, zxid_idp_soap, zxid_sp_deref_art, zxid_sp_soap */
+struct zx_root_s* zxid_soap_call_body(zxid_conf* cf, struct zx_str* url, struct zx_e_Body_s* body)
 {
   /*return zxid_soap_call_hdr_body(cf, url, zx_NEW_e_Header(cf->ctx), body);*/
   return zxid_soap_call_hdr_body(cf, url, 0, body);
 }
 
 /*() Emit to stdout XML data structure representing SOAP envelope (request).
- * Typically used in CGI environment.
+ * Typically used in CGI environment, e.g. by the IdP and Discovery.
+ * Optionally logs the issued message to local audit trail.
  *
  * cf::     ZXID configuration object, also used for memory allocation
  * body::   XML data structure representing the request
  * return:: 0 if fail, ZXID_REDIR_OK if success. */
 
-/* Called by:  zxid_idp_soap_dispatch x2, zxid_sp_soap_dispatch x5 */
-int zxid_soap_cgi_resp_body(struct zxid_conf* cf, struct zx_e_Body_s* body)
+/* Called by:  zxid_idp_soap_dispatch x2, zxid_sp_soap_dispatch x6 */
+int zxid_soap_cgi_resp_body(zxid_conf* cf, struct zx_e_Body_s* body, struct zx_str* entid)
 {
   struct zx_e_Envelope_s* env = zx_NEW_e_Envelope(cf->ctx);
   struct zx_str* ss;
+  struct zx_str* logpath;
   
   env->Header = zx_NEW_e_Header(cf->ctx);
   env->Body = body;
   ss = zx_EASY_ENC_SO_e_Envelope(cf->ctx, env);
 
+  if (cf->log_issue_msg) {
+    logpath = zxlog_path(cf, entid, ss, ZXLOG_ISSUE_DIR, ZXLOG_WIR_KIND, 1);
+    if (logpath) {
+      if (zxlog_dup_check(cf, logpath, "cgi_resp")) {
+	ERR("Duplicate wire msg(%.*s) (Simple Sign)", ss->len, ss->s);
+#if 0
+	if (cf->dup_msg_fatal) {
+	  ERR("FATAL (by configuration): Duplicate wire msg(%.*s) (cgi_resp)", ss->len, ss->s);
+	  zxlog_blob(cf, 1, logpath, ss, "cgi_resp dup");
+	  zx_str_free(cf->ctx, logpath);
+	  return 0;
+	}
+#endif
+      }
+      zxlog_blob(cf, 1, logpath, ss, "cgi_resp");
+      zxlog(cf, 0, 0, 0, entid, 0, 0, 0, "N", "K", "CGIRESP", 0, "logpath(%.*s)", logpath->len, logpath->s);
+      zx_str_free(cf->ctx, logpath);
+    }
+  }
+  
   if (zx_debug & ZXID_INOUT) INFO("SOAP_RESP(%.*s)", ss->len, ss->s);
   printf("CONTENT-TYPE: text/xml" CRLF "CONTENT-LENGTH: %d" CRLF2 "%.*s", ss->len, ss->len, ss->s);
   return ZXID_REDIR_OK;
@@ -207,13 +231,14 @@ int zxid_soap_cgi_resp_body(struct zxid_conf* cf, struct zx_e_Body_s* body)
  * payload::     What should be encoded in the redirect URL. Effectively becomes the query string
  * relay_state:: Optional relay state argument. Ends up being encoded in the query string
  * sign::        Whether binding layer signature is to be applied: 0=no, 1=POST-Simple-Sign
- * url::         URL where the form should be posted
+ * action_url::  URL where the form should be posted
  * return::      Query string encoding of the request. The memory should be freed by the caller.
  *     0 on failure.  */
 
 /* Called by:  zxid_idp_sso x3 */
-struct zx_str* zxid_saml2_post_enc(struct zxid_conf* cf, char* field, struct zx_str* payload, char* relay_state, int sign, struct zx_str* action_url)
+struct zx_str* zxid_saml2_post_enc(zxid_conf* cf, char* field, struct zx_str* payload, char* relay_state, int sign, struct zx_str* action_url)
 {
+  RSA* sign_pkey;
   struct zx_str id_str;
   struct zx_str* logpath;
   char* sigbuf[SIG_SIZE];
@@ -249,9 +274,8 @@ struct zx_str* zxid_saml2_post_enc(struct zxid_conf* cf, char* field, struct zx_
     memcpy(p, "&SigAlg=" SIG_ALGO, sizeof("&SigAlg=" SIG_ALGO)-1);
     p += sizeof("&SigAlg=" SIG_ALGO)-1;
 
-    if (!cf->sign_pkey)
-      cf->sign_pkey = zxid_read_private_key(cf, "sign-nopw-cert.pem");
-    zlen = zxsig_data_rsa_sha1(cf->ctx, p-url, url, &zbuf, cf->sign_pkey, "SAML2 post");
+    if (zxid_lazy_load_sign_cert_and_pkey(cf, 0, &sign_pkey, "SAML2 post"))
+      zlen = zxsig_data_rsa_sha1(cf->ctx, p-url, url, &zbuf, sign_pkey, "SAML2 post");
     if (zlen == -1)
       return 0;
 
@@ -334,8 +358,9 @@ struct zx_str zxstr_unknown = { {0,0,0,0,0}, sizeof("UNKNOWN")-1, "UNKNOWN" };
  * return::      Query string encoding of the request. The memory should be freed by the caller. */
 
 /* Called by:  zxid_saml2_redir, zxid_saml2_redir_url, zxid_saml2_resp_redir */
-struct zx_str* zxid_saml2_redir_enc(struct zxid_conf* cf, char* field, struct zx_str* pay_load, char* relay_state)
+struct zx_str* zxid_saml2_redir_enc(zxid_conf* cf, char* field, struct zx_str* pay_load, char* relay_state)
 {
+  RSA* sign_pkey;
   struct zx_str* logpath;
   struct zx_str* ss;
   char* zbuf;
@@ -379,9 +404,8 @@ struct zx_str* zxid_saml2_redir_enc(struct zxid_conf* cf, char* field, struct zx
   
   memcpy(url+len, "&SigAlg=" SIG_ALGO_URLENC, sizeof("&SigAlg=" SIG_ALGO_URLENC)-1);
   len += sizeof("&SigAlg=" SIG_ALGO_URLENC)-1;
-  if (!cf->sign_pkey)
-    cf->sign_pkey = zxid_read_private_key(cf, "sign-nopw-cert.pem");
-  zlen = zxsig_data_rsa_sha1(cf->ctx, len, url, &zbuf, cf->sign_pkey, "SAML2 redir");
+  if (zxid_lazy_load_sign_cert_and_pkey(cf, 0, &sign_pkey, "SAML2 redir"))
+    zlen = zxsig_data_rsa_sha1(cf->ctx, len, url, &zbuf, sign_pkey, "SAML2 redir");
   if (zlen == -1)
     return 0;
   
@@ -436,7 +460,7 @@ struct zx_str* zxid_saml2_redir_enc(struct zxid_conf* cf, char* field, struct zx
  * return::      URL suitable for redirection as ~zx_str~. The memory should be freed by the caller. */
 
 /* Called by:  zxid_start_sso_url */
-struct zx_str* zxid_saml2_redir_url(struct zxid_conf* cf, struct zx_str* loc, struct zx_str* pay_load, char* relay_state)
+struct zx_str* zxid_saml2_redir_url(zxid_conf* cf, struct zx_str* loc, struct zx_str* pay_load, char* relay_state)
 {
   struct zx_str* ss;
   struct zx_str* rse = zxid_saml2_redir_enc(cf, "SAMLRequest=", pay_load, relay_state);
@@ -464,7 +488,7 @@ struct zx_str* zxid_saml2_redir_url(struct zxid_conf* cf, struct zx_str* loc, st
  * return::      HTTP Location header as ~zx_str~. The memory should be freed by the caller. */
 
 /* Called by:  zxid_sp_mni_redir, zxid_sp_slo_redir */
-struct zx_str* zxid_saml2_redir(struct zxid_conf* cf, struct zx_str* loc, struct zx_str* pay_load, char* relay_state)
+struct zx_str* zxid_saml2_redir(zxid_conf* cf, struct zx_str* loc, struct zx_str* pay_load, char* relay_state)
 {
   struct zx_str* ss;
   struct zx_str* rse = zxid_saml2_redir_enc(cf, "SAMLRequest=", pay_load, relay_state);
@@ -491,7 +515,7 @@ struct zx_str* zxid_saml2_redir(struct zxid_conf* cf, struct zx_str* loc, struct
  * return::      HTTP Location header as ~zx_str~. The memory should be freed by the caller. */
 
 /* Called by:  zxid_idp_dispatch, zxid_slo_resp_redir, zxid_sp_dispatch */
-struct zx_str* zxid_saml2_resp_redir(struct zxid_conf* cf, struct zx_str* loc, struct zx_str* pay_load, char* relay_state)
+struct zx_str* zxid_saml2_resp_redir(zxid_conf* cf, struct zx_str* loc, struct zx_str* pay_load, char* relay_state)
 {
   struct zx_str* ss;
   struct zx_str* rse = zxid_saml2_redir_enc(cf, "SAMLResponse=", pay_load, relay_state);
@@ -517,7 +541,7 @@ struct zx_str* zxid_saml2_resp_redir(struct zxid_conf* cf, struct zx_str* loc, s
  * return:: 1 of SAML message is OK, 0 if message is not OK. */
 
 /* Called by:  zxid_idp_dispatch x2, zxid_idp_soap_dispatch, zxid_pep_az_soap, zxid_sp_dispatch x3, zxid_sp_mni_soap, zxid_sp_slo_soap, zxid_sp_soap_dispatch x3 */
-int zxid_saml_ok(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zx_sp_Status_s* st, char* what)
+int zxid_saml_ok(zxid_conf* cf, zxid_cgi* cgi, struct zx_sp_Status_s* st, char* what)
 {
   struct zx_str* ss;
   struct zx_str* m = 0;
@@ -568,8 +592,8 @@ int zxid_saml_ok(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zx_sp_Status
  *     structure is decrypted and its contents returned as the Name ID
  * return:: XML data structure corresponding to (possibly decrypted) Name ID */
 
-/* Called by:  test_ibm_cert_problem, test_ibm_cert_problem_enc_dec, zxid_di_query, zxid_idp_slo_do, zxid_mni_do, zxid_sp_slo_do, zxid_sp_sso_finalize */
-struct zx_sa_NameID_s* zxid_decrypt_nameid(struct zxid_conf* cf, struct zx_sa_NameID_s* nid, struct zx_sa_EncryptedID_s* encid)
+/* Called by:  test_ibm_cert_problem, test_ibm_cert_problem_enc_dec, zxid_di_query, zxid_idp_slo_do, zxid_mni_do, zxid_sp_slo_do, zxid_sp_sso_finalize, zxid_wsf_validate_a7n */
+zxid_nid* zxid_decrypt_nameid(zxid_conf* cf, zxid_nid* nid, struct zx_sa_EncryptedID_s* encid)
 {
   struct zx_str* ss;
   struct zx_root_s* r;
@@ -581,8 +605,10 @@ struct zx_sa_NameID_s* zxid_decrypt_nameid(struct zxid_conf* cf, struct zx_sa_Na
       ERR("Failed to decrypt NameID. Most probably certificate-private key mismatch or metadata problem. Could also be corrupt message. %d", 0);
       return 0;
     }
+    LOCK(cf->ctx->mx, "dec nid");
     zx_prepare_dec_ctx(cf->ctx, zx_ns_tab, ss->s, ss->s + ss->len);
     r = zx_DEC_root(cf->ctx, 0, 1);
+    UNLOCK(cf->ctx->mx, "dec nid");
     if (!r) {
       ERR("Failed to parse EncryptedID buf(%.*s)", ss->len, ss->s);
       return 0;
@@ -603,7 +629,7 @@ struct zx_sa_NameID_s* zxid_decrypt_nameid(struct zxid_conf* cf, struct zx_sa_Na
  * return:: XML data structure corresponding to (possibly decrypted) new Name ID */
 
 /* Called by:  zxid_mni_do */
-struct zx_str* zxid_decrypt_newnym(struct zxid_conf* cf, struct zx_str* newnym, struct zx_sp_NewEncryptedID_s* encid)
+struct zx_str* zxid_decrypt_newnym(zxid_conf* cf, struct zx_str* newnym, struct zx_sp_NewEncryptedID_s* encid)
 {
   struct zx_str* ss;
   struct zx_root_s* r;
@@ -611,8 +637,10 @@ struct zx_str* zxid_decrypt_newnym(struct zxid_conf* cf, struct zx_str* newnym,
     return newnym;
   if (encid) {
     ss = zxenc_privkey_dec(cf, encid->EncryptedData, encid->EncryptedKey);
+    LOCK(cf->ctx->mx, "dec newnym");
     zx_prepare_dec_ctx(cf->ctx, zx_ns_tab, ss->s, ss->s + ss->len);
     r = zx_DEC_root(cf->ctx, 0, 1);
+    UNLOCK(cf->ctx->mx, "dec newnym");
     if (!r) {
       ERR("Failed to parse NewEncryptedID buf(%.*s)", ss->len, ss->s);
       return 0;
@@ -647,12 +675,12 @@ struct zx_str* zxid_decrypt_newnym(struct zxid_conf* cf, struct zx_str* newnym,
  * manually and then call zxsig_validate() with correctly populate refs array.
  */
 
-/* Called by:  zxid_idp_slo_do, zxid_mni_do, zxid_sp_dig_sso_a7n, zxid_sp_slo_do, zxid_xacml_az_do */
-int zxid_chk_sig(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zxid_ses* ses, struct zx_elem_s* elem, struct zx_ds_Signature_s* sig, struct zx_sa_Issuer_s* issue_ent, const char* lk)
+/* Called by:  zxid_idp_slo_do, zxid_mni_do, zxid_sp_dig_sso_a7n, zxid_sp_slo_do, zxid_xacml_az_cd1_do, zxid_xacml_az_do */
+int zxid_chk_sig(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, struct zx_elem_s* elem, struct zx_ds_Signature_s* sig, struct zx_sa_Issuer_s* issue_ent, const char* lk)
 {
   struct zx_str* issuer = 0;
   struct zxsig_ref refs;
-  struct zxid_entity* idp_meta;
+  zxid_entity* idp_meta;
   char* err = "S"; /* See: RES in zxid-log.pd, section "ZXID Log Format" */
   
   if (!sig) { D("No signature in %s", lk); return 1; /* Not an error */ }
@@ -708,7 +736,7 @@ erro:
 /*() Transform content according to map. The returned zx_str will be nul terminated. */
 
 /* Called by:  pool2apache x2, zxid_add_at_values, zxid_pep_az_soap x2, zxid_pool_to_json x2, zxid_pool_to_ldif x2, zxid_pool_to_qs x2 */
-struct zx_str* zxid_map_val(struct zxid_conf* cf, struct zxid_map* map, struct zx_str* val)
+struct zx_str* zxid_map_val(zxid_conf* cf, struct zxid_map* map, struct zx_str* val)
 {
   struct zx_str* ss = val;
   char* bin;
@@ -771,4 +799,46 @@ struct zx_str* zxid_map_val(struct zxid_conf* cf, struct zxid_map* map, struct z
   return ss;
 }
 
+/*() Extract from a string representing SOAP envelope, the payload part in the body. */
+
+/* Called by:  zxcall_main */
+char* zxid_extract_body(zxid_conf* cf, char* enve)
+{
+  char* p;
+  char* q;
+
+  if (!p)
+    goto nobody;
+  for (p = enve; p; p+=4) {
+    p = strstr(p, "Body");
+    if (!p) {
+nobody:
+      ERR("Response does not contain <Body> res(%s)", STRNULLCHKD(enve));
+      return 0;
+    }
+    if (p > enve && ONE_OF_2(p[-1], '<', ':') && ONE_OF_5(p[4], '>', ' ', '\t', '\r', '\n'))
+      break; /* Opening <Body> detected. */
+  }
+  if (!p)
+    goto nobody;
+  
+  for (p += 4; *p && *p != '>'; ++p) ;  /* Scan for close of opening <Body */
+  if (!*p)
+    goto nobody;
+  
+  for (q = ++p; q; q+=5) {
+    q = strstr(q, "Body>");
+    if (!q)
+      goto nobody;  /* Missing closing </Body> tag */
+    if (ONE_OF_2(q[-1], '<', ':'))
+      break;
+  }
+  for (--q; *q != '<'; --q) ;  /* Scan for the start of </Body>, skipping any namespace prefix */
+
+  enve = ZX_ALLOC(cf->ctx, q-p+1);
+  memcpy(enve, p, q-p);
+  enve[q-p] = 0;
+  return enve;
+}
+
 /* EOF  --  zxidlib.c */
