@@ -1,4 +1,5 @@
 /* zxidspx.c  -  Handwritten functions for SP dispatch
+ * Copyright (c) 2010 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
  * Copyright (c) 2006-2009 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
  * This is confidential unpublished proprietary source code of the author.
@@ -13,6 +14,7 @@
  * 7.10.2008,  added documentation --Sampo
  * 22.8.2009,  added XACML dummy PDP support --Sampo
  * 15.11.2009, added discovery service Query --Sampo
+ * 12.2.2010,  added locking to lazy loading --Sampo
  *
  * See also zxid/sg/wsf-soap11.sg and zxid/c/zx-e-data.h, which is generated.
  */
@@ -29,8 +31,8 @@
 
 /*() Extract an assertion, decrypting EncryptedAssertion if needed. */
 
-/* Called by:  zxid_sp_dig_sso_a7n, zxid_sp_soap_dispatch */
-struct zx_sa_Assertion_s* zxid_dec_a7n(struct zxid_conf* cf, struct zx_sa_Assertion_s* a7n, struct zx_sa_EncryptedAssertion_s* enca7n)
+/* Called by:  zxid_sp_dig_sso_a7n, zxid_sp_soap_dispatch, zxid_wsp_validate */
+zxid_a7n* zxid_dec_a7n(zxid_conf* cf, zxid_a7n* a7n, struct zx_sa_EncryptedAssertion_s* enca7n)
 {
   struct zx_str* ss;
   struct zx_root_s* r;
@@ -40,8 +42,10 @@ struct zx_sa_Assertion_s* zxid_dec_a7n(struct zxid_conf* cf, struct zx_sa_Assert
     if (!ss || !ss->s || !ss->len) {
       return 0;
     }
+    LOCK(cf->ctx->mx, "dec a7n");
     zx_prepare_dec_ctx(cf->ctx, zx_ns_tab, ss->s, ss->s + ss->len);
     r = zx_DEC_root(cf->ctx, 0, 1);
+    UNLOCK(cf->ctx->mx, "dec a7n");
     if (!r) {
       ERR("Failed to parse EncryptedAssertion buf(%.*s)", ss->len, ss->s);
       zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "C", "BADXML", 0, "bad EncryptedAssertion");
@@ -55,9 +59,9 @@ struct zx_sa_Assertion_s* zxid_dec_a7n(struct zxid_conf* cf, struct zx_sa_Assert
 /*() Extract an assertion from Request, decrypting EncryptedAssertion if needed, and perform SSO */
 
 /* Called by:  zxid_idp_soap_dispatch, zxid_sp_dispatch, zxid_sp_soap_dispatch x3 */
-static int zxid_sp_dig_sso_a7n(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zxid_ses* ses, struct zx_sp_Response_s* resp)
+static int zxid_sp_dig_sso_a7n(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, struct zx_sp_Response_s* resp)
 {
-  struct zx_sa_Assertion_s* a7n;
+  zxid_a7n* a7n;
 
   if (!zxid_chk_sig(cf, cgi, ses, (struct zx_elem_s*)resp, resp->Signature, resp->Issuer, "Response"))
     return 0;
@@ -83,10 +87,10 @@ static int zxid_sp_dig_sso_a7n(struct zxid_conf* cf, struct zxid_cgi* cgi, struc
  *     strings such as "Location: ..." should be freed by caller. */
 
 /* Called by:  main x3, zxid_mgmt, zxid_simple_no_ses_cf, zxid_simple_ses_active_cf */
-struct zx_str* zxid_sp_dispatch(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zxid_ses* ses)
+struct zx_str* zxid_sp_dispatch(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses)
 {
   struct zx_sp_LogoutRequest_s* req;
-  struct zxid_entity* idp_meta;
+  zxid_entity* idp_meta;
   struct zx_str* loc;
   struct zx_str* ss;
   struct zx_str* ss2;
@@ -106,7 +110,7 @@ struct zx_str* zxid_sp_dispatch(struct zxid_conf* cf, struct zxid_cgi* cgi, stru
     switch (ret) {
     case ZXID_OK:     return zx_dup_str(cf->ctx, "K");
     case ZXID_SSO_OK: return zx_dup_str(cf->ctx, "O");
-    case ZXID_FAIL:   D("*** FAIL, should send back to IdP select %d", 0);
+    case ZXID_FAIL:   D("*** FAIL, should send back to IdP select %d", 0); return zx_dup_str(cf->ctx, "* ERR");
     }
     return zx_dup_str(cf->ctx, "M");
   }
@@ -166,7 +170,7 @@ struct zx_str* zxid_sp_dispatch(struct zxid_conf* cf, struct zxid_cgi* cgi, stru
 /*() Create Authorization Decision */
 
 /* Called by:  zxid_xacml_az_do x2 */
-static void zxid_ins_xacml_az_stmt(struct zxid_conf* cf, struct zx_sa_Assertion_s* a7n, char* deci)
+static void zxid_ins_xacml_az_stmt(zxid_conf* cf, zxid_a7n* a7n, char* deci)
 {
   /* Two ways of doing assertion with XACMLAuthzDecisionStatement:
    * 1. Explicitly include such statement in assertion
@@ -185,14 +189,34 @@ static void zxid_ins_xacml_az_stmt(struct zxid_conf* cf, struct zx_sa_Assertion_
 #endif
 }
 
+/* Called by:  zxid_xacml_az_cd1_do x2 */
+static void zxid_ins_xacml_az_cd1_stmt(zxid_conf* cf, zxid_a7n* a7n, char* deci)
+{
+  /* Two ways of doing assertion with XACMLAuthzDecisionStatement:
+   * 1. Explicitly include such statement in assertion
+   * 2. Use sa:Statement, but brandit with xsi:type
+   * The former is more logical, but the latter is what Jericho does
+   * and in effect the XACML interop events have done (de-facto standard?). */
+
+#if 1
+  a7n->xasacd1_XACMLAuthzDecisionStatement = zx_NEW_xasacd1_XACMLAuthzDecisionStatement(cf->ctx);
+  a7n->xasacd1_XACMLAuthzDecisionStatement->Response = zxid_mk_xacml_resp(cf, deci);
+  /* *** Add xaspcd1 and xasacd1 variants */
+#else
+  a7n->Statement = zx_NEW_sa_Statement(cf->ctx);
+  a7n->Statement->type = zx_ref_str(cf->ctx, "xasacd1:XACMLAuthzDecisionStatementType");
+  a7n->Statement->Response = zxid_mk_xacml_resp(cf, deci);
+#endif
+}
+
 /*() Process <XACMLAuthzDecisionQuery>. The response will have
  * SAML assertion containing Authorization Decision Statement. */
 
 /* Called by:  zxid_sp_soap_dispatch */
-static struct zx_sp_Response_s* zxid_xacml_az_do(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zxid_ses* ses, struct zx_xasp_XACMLAuthzDecisionQuery_s* azq)
+static struct zx_sp_Response_s* zxid_xacml_az_do(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, struct zx_xasp_XACMLAuthzDecisionQuery_s* azq)
 {
   struct zx_sp_Response_s* resp;
-  struct zx_sa_Assertion_s* a7n;
+  zxid_a7n* a7n;
   struct zx_str* affil;
   struct zx_str* subj;
   struct zx_xac_Attribute_s* xac_at;
@@ -226,12 +250,58 @@ static struct zx_sp_Response_s* zxid_xacml_az_do(struct zxid_conf* cf, struct zx
       }
     }
   }
+  D("PDP: Permit by default %d",0);
   zxid_ins_xacml_az_stmt(cf, a7n, "Permit");
   resp = zxid_mk_saml_resp(cf);
   resp->Assertion = a7n;
   return resp;
 }
 
+/* Called by:  zxid_sp_soap_dispatch */
+static struct zx_sp_Response_s* zxid_xacml_az_cd1_do(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, struct zx_xaspcd1_XACMLAuthzDecisionQuery_s* azq)
+{
+  struct zx_sp_Response_s* resp;
+  zxid_a7n* a7n;
+  struct zx_str* affil;
+  struct zx_str* subj;
+  struct zx_xac_Attribute_s* xac_at;
+  
+  if (!zxid_chk_sig(cf, cgi, ses, (struct zx_elem_s*)azq, azq->Signature, azq->Issuer, "XACMLAuthzDecisionQuery"))
+    return 0;
+
+  affil = subj = 0;
+#if 0
+  affil = ar->NameIDPolicy && ar->NameIDPolicy->SPNameQualifier
+    ? ar->NameIDPolicy->SPNameQualifier
+    : ar->Issuer->gg.content;
+  subj = zxid_mk_subj(cf, ses, affil, sp_meta);
+#endif
+  //a7n = zxid_mk_a7n(cf, affil, subj, 0, 0, 0);
+  a7n = zxid_mk_a7n(cf, affil, 0, 0, 0, 0);
+
+  if (azq->Request && azq->Request->Subject) {
+    for (xac_at = azq->Request->Subject->Attribute; xac_at; xac_at = (struct zx_xac_Attribute_s*)ZX_NEXT(xac_at)) {
+      if (xac_at->AttributeId->len == sizeof("role")-1
+	  && !memcmp(xac_at->AttributeId->s, "role", sizeof("role")-1)) {
+	if (xac_at->AttributeValue && xac_at->AttributeValue->content
+	    && xac_at->AttributeValue->content->len == sizeof("deny")-1
+	    && !memcmp(xac_at->AttributeValue->content->s, "deny", sizeof("deny")-1)) {
+	  D("PDP: Deny due to role=deny %d",0);
+	  zxid_ins_xacml_az_cd1_stmt(cf, a7n, "Deny");
+	  resp = zxid_mk_saml_resp(cf);
+	  resp->Assertion = a7n;
+	  return resp;
+	}
+      }
+    }
+  }
+  D("PDP: Permit by default %d",0);
+  zxid_ins_xacml_az_cd1_stmt(cf, a7n, "Permit");
+  resp = zxid_mk_saml_resp(cf);
+  resp->Assertion = a7n;
+  return resp;
+}
+
 /*() SOAP dispatch can also handle requests and responses received via artifact
  * resolution. However only some combinations make sense.
  * See zxid/sg/wsf-soap11.sg for the master SOAP dispatch from parsing perspective.
@@ -239,11 +309,14 @@ static struct zx_sp_Response_s* zxid_xacml_az_do(struct zxid_conf* cf, struct zx
  * Return 0 for failure, otherwise some success code such as ZXID_SSO_OK */
 
 /* Called by:  zxid_idp_soap_parse, zxid_sp_deref_art, zxid_sp_soap_parse */
-int zxid_sp_soap_dispatch(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zxid_ses* ses, struct zx_root_s* r)
+int zxid_sp_soap_dispatch(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, struct zx_root_s* r)
 {
+  X509* sign_cert;
+  RSA*  sign_pkey;
   struct zxsig_ref refs;
   struct zx_e_Body_s* body;
   struct zx_sp_LogoutRequest_s* req;
+  struct zx_str* issuer;
   ses->sigres = ZXSIG_NO_SIG;
 
   if (!r) goto bad;
@@ -273,14 +346,11 @@ int zxid_sp_soap_dispatch(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zxi
     if (cf->sso_soap_resp_sign) {
       refs.id = body->LogoutResponse->ID;
       refs.canon = zx_EASY_ENC_SO_sp_LogoutResponse(cf->ctx, body->LogoutResponse);
-      if (!cf->sign_cert) // Lazy load cert and private key
-	cf->sign_cert = zxid_read_cert(cf, "sign-nopw-cert.pem");
-      if (!cf->sign_pkey)
-	cf->sign_pkey = zxid_read_private_key(cf, "sign-nopw-cert.pem");
-      body->LogoutResponse->Signature = zxsig_sign(cf->ctx, 1, &refs, cf->sign_cert, cf->sign_pkey);
+      if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert slor"))
+	body->LogoutResponse->Signature = zxsig_sign(cf->ctx, 1, &refs, sign_cert, sign_pkey);
       zx_str_free(cf->ctx, refs.canon);
     }
-    return zxid_soap_cgi_resp_body(cf, body);
+    return zxid_soap_cgi_resp_body(cf, body, req->Issuer->gg.content);
   }
 
   if (r->Envelope->Body->ManageNameIDRequest) {
@@ -291,20 +361,23 @@ int zxid_sp_soap_dispatch(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zxi
     if (cf->sso_soap_resp_sign) {
       refs.id = res->ID;
       refs.canon = zx_EASY_ENC_SO_sp_ManageNameIDResponse(cf->ctx, res);
-      if (!cf->sign_cert) // Lazy load cert and private key
-	cf->sign_cert = zxid_read_cert(cf, "sign-nopw-cert.pem");
-      if (!cf->sign_pkey)
-	cf->sign_pkey = zxid_read_private_key(cf, "sign-nopw-cert.pem");
-      res->Signature = zxsig_sign(cf->ctx, 1, &refs, cf->sign_cert, cf->sign_pkey);
+      if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert mnir"))
+	res->Signature = zxsig_sign(cf->ctx, 1, &refs, sign_cert, sign_pkey);
       zx_str_free(cf->ctx, refs.canon);
     }
-    return zxid_soap_cgi_resp_body(cf, body);
+    return zxid_soap_cgi_resp_body(cf, body, r->Envelope->Body->ManageNameIDRequest->Issuer->gg.content);
   }
 
   D("as_ena=%d %p", cf->as_ena, r->Envelope->Body->SASLRequest);
   if (cf->as_ena) {
     if (r->Envelope->Body->SASLRequest) {
       struct zx_as_SASLResponse_s* res;
+      if (r->Envelope->Header && r->Envelope->Header->Sender
+	  && r->Envelope->Header->Sender->providerID)
+	issuer = r->Envelope->Header->Sender->providerID;
+      else
+	issuer = 0;
+      //issuer = r->Envelope->Body->SASLRequest->Issuer->gg.content;
       res = zxid_idp_as_do(cf, r->Envelope->Body->SASLRequest);
       body = zx_NEW_e_Body(cf->ctx);
       body->SASLResponse = res;
@@ -312,15 +385,12 @@ int zxid_sp_soap_dispatch(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zxi
       if (cf->sso_soap_resp_sign) {
 	refs.id = res->ID;
 	refs.canon = zx_EASY_ENC_SO_as_SASLResponse(cf->ctx, res);
-	if (!cf->sign_cert) // Lazy load cert and private key
-	  cf->sign_cert = zxid_read_cert(cf, "sign-nopw-cert.pem");
-	if (!cf->sign_pkey)
-	  cf->sign_pkey = zxid_read_private_key(cf, "sign-nopw-cert.pem");
-	res->Signature = zxsig_sign(cf->ctx, 1, &refs, cf->sign_cert, cf->sign_pkey);
+	if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert asr"))
+	  res->Signature = zxsig_sign(cf->ctx, 1, &refs, sign_cert, sign_pkey);
 	zx_str_free(cf->ctx, refs.canon);
       }
 #endif
-      return zxid_soap_cgi_resp_body(cf, body);
+      return zxid_soap_cgi_resp_body(cf, body, issuer);
     }
   }
   
@@ -333,13 +403,16 @@ int zxid_sp_soap_dispatch(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zxi
     }
 
     if (r->Envelope->Body->Query) { /* Discovery 2.0 Query */
-      struct zx_sa_Assertion_s* a7n;
+      zxid_a7n* a7n;
       struct zx_di_QueryResponse_s* di_resp;
 
-      if (!r->Envelope->Header || !r->Envelope->Header->Security) {
+      if (!r->Envelope->Header || !r->Envelope->Header->Security
+	  || !r->Envelope->Header->Sender || !r->Envelope->Header->Sender->providerID) {
 	goto malformed;
       }
-
+      
+      issuer = r->Envelope->Header->Sender->providerID;
+      
       a7n = zxid_dec_a7n(cf, r->Envelope->Header->Security->Assertion,
 			 r->Envelope->Header->Security->EncryptedAssertion);
       di_resp = zxid_di_query(cf, a7n, r->Envelope->Body->Query);
@@ -350,15 +423,12 @@ int zxid_sp_soap_dispatch(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zxi
       if (cf->sso_soap_resp_sign) {
 	refs.id = di_resp->ID;
 	refs.canon = zx_EASY_ENC_SO_e_Body(cf->ctx, body);
-	if (!cf->sign_cert) // Lazy load cert and private key
-	  cf->sign_cert = zxid_read_cert(cf, "sign-nopw-cert.pem");
-	if (!cf->sign_pkey)
-	  cf->sign_pkey = zxid_read_private_key(cf, "sign-nopw-cert.pem");
-	res->Signature = zxsig_sign(cf->ctx, 1, &refs, cf->sign_cert, cf->sign_pkey);
+	if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert dir"))
+	  res->Signature = zxsig_sign(cf->ctx, 1, &refs, sign_cert, sign_pkey);
 	zx_str_free(cf->ctx, refs.canon);
       }
 #endif
-      return zxid_soap_cgi_resp_body(cf, body);
+      return zxid_soap_cgi_resp_body(cf, body, issuer);
     }
   }
   
@@ -372,14 +442,26 @@ int zxid_sp_soap_dispatch(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zxi
       if (cf->sso_soap_resp_sign) {
 	refs.id = res->ID;
 	refs.canon = zx_EASY_ENC_SO_sp_Response(cf->ctx, res);
-	if (!cf->sign_cert) // Lazy load cert and private key
-	  cf->sign_cert = zxid_read_cert(cf, "sign-nopw-cert.pem");
-	if (!cf->sign_pkey)
-	  cf->sign_pkey = zxid_read_private_key(cf, "sign-nopw-cert.pem");
-	res->Signature = zxsig_sign(cf->ctx, 1, &refs, cf->sign_cert, cf->sign_pkey);
+	if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert azr"))
+	res->Signature = zxsig_sign(cf->ctx, 1, &refs, sign_cert, sign_pkey);
 	zx_str_free(cf->ctx, refs.canon);
       }
-      return zxid_soap_cgi_resp_body(cf, body);
+      return zxid_soap_cgi_resp_body(cf, body, r->Envelope->Body->XACMLAuthzDecisionQuery->Issuer->gg.content);
+    }
+    if (r->Envelope->Body->xaspcd1_XACMLAuthzDecisionQuery) {
+      struct zx_sp_Response_s* res;
+      D("xaspcd1:XACMLAuthzDecisionQuery %d",0);
+      res = zxid_xacml_az_cd1_do(cf, cgi, ses, r->Envelope->Body->xaspcd1_XACMLAuthzDecisionQuery);
+      body = zx_NEW_e_Body(cf->ctx);
+      body->Response = res;
+      if (cf->sso_soap_resp_sign) {
+	refs.id = res->ID;
+	refs.canon = zx_EASY_ENC_SO_sp_Response(cf->ctx, res);
+	if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert azr"))
+	res->Signature = zxsig_sign(cf->ctx, 1, &refs, sign_cert, sign_pkey);
+	zx_str_free(cf->ctx, refs.canon);
+      }
+      return zxid_soap_cgi_resp_body(cf, body, r->Envelope->Body->XACMLAuthzDecisionQuery->Issuer->gg.content);
     }
   }
   
@@ -398,11 +480,13 @@ int zxid_sp_soap_dispatch(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zxi
 /*() Return 0 for failure, otherwise some success code such as ZXID_SSO_OK */
 
 /* Called by:  chkuid, main x6, zxid_simple_cf_ses */
-int zxid_sp_soap_parse(struct zxid_conf* cf, struct zxid_cgi* cgi, struct zxid_ses* ses, int len, char* buf)
+int zxid_sp_soap_parse(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, int len, char* buf)
 {
   struct zx_root_s* r;
+  LOCK(cf->ctx->mx, "sp soap parse");
   zx_prepare_dec_ctx(cf->ctx, zx_ns_tab, buf, buf + len);
   r = zx_DEC_root(cf->ctx, 0, 1);
+  UNLOCK(cf->ctx->mx, "sp soap parse");
   if (!r || !r->Envelope || !r->Envelope->Body) {
     ERR("Failed to parse SOAP request buf(%.*s)", len, buf);
     zxlog(cf, 0, 0, 0, 0, 0, 0, ses->nameid?ses->nameid->gg.content:0, "N", "C", "BADXML", 0, "sid(%s) bad soap req", STRNULLCHK(ses->sid));
